import 'package:flutter/material.dart';
import 'package:hometec_ui/components/progress_bar/loading_progress_bar.dart';
import 'package:hometec_ui/components/state_list/model/state_list_model.dart';
import 'package:hometec_ui/measure_size.dart';
import 'package:hometec_ui/resources/colors/color.dart';
import 'package:hometec_ui/resources/fonts/font.dart';
import 'package:rxdart/subjects.dart';

class StateList extends StatefulWidget {

  final Widget child;
  final Stream<StateListEnum> state;
  final String emptyMessage;
  final String errorMessage;
  final String networkMessage;
  final Widget other;

  const StateList({Key key, @required this.child, @required this.state, @required this.emptyMessage, @required this.errorMessage, this.other, this.networkMessage}) : super(key: key);

  @override
  _StateListState createState() => _StateListState(child, state, other);
}

class _StateListState extends State<StateList> {
  final Widget child;
  final Stream<StateListEnum> state;
  final Widget other;
  final BehaviorSubject<double> _heightController = BehaviorSubject<double>.seeded(0);
  Stream<double> get outHieght => _heightController.stream;

  _StateListState(this.child, this.state, this.other);

  @override
  Widget build(BuildContext context) {
    return MeasureSize(
      onChange: (size) => _heightController.add(size.height),
      child: StreamBuilder(
      stream: state,
      builder: (context, snapshot){
        if(snapshot.hasData)
          switch (snapshot.data) {
            case StateListEnum.success:
              return Container(
                child: child
              );
            case StateListEnum.other:
              return Container(
                child: other
              );
            case StateListEnum.empty:
              return StreamBuilder<double>(
                builder: (context, snapshot){
                  return Container(
                    height: snapshot.data,
                    child: Padding(
                      padding: EdgeInsets.all(24),
                      child: Center(
                        child: Text(widget.emptyMessage, textAlign: TextAlign.center, style: Font.bodyRegular.style(color: ECColor.textColor),),
                      )
                    ),
                  );
                }
              );
            case StateListEnum.error:
              return StreamBuilder<double>(
                builder: (context, snapshot){
                  return Container(
                    height: snapshot.data,
                    child: Padding(
                      padding: EdgeInsets.all(24),
                        child: Column(
                          mainAxisSize: MainAxisSize.min,
                          children: <Widget>[
                            Icon(Icons.report_problem, size: 72, color: ECColor.textColor),
                            SizedBox(height: 8),
                            Text(widget.errorMessage, textAlign: TextAlign.center, style: Font.bodyRegular.style(color: ECColor.textColor),),
                          ],
                        )
                      ),
                  );
                }
              );             
            case StateListEnum.network:
              return StreamBuilder<double>(
                builder: (context, snapshot){
                  return Container(
                    height: snapshot.data,
                    child: Padding(
                      padding: EdgeInsets.all(24),
                        child: Column(
                          mainAxisSize: MainAxisSize.min,
                          children: <Widget>[
                            Icon(Icons.signal_wifi_off, size: 72, color: ECColor.textColor),
                            SizedBox(height: 8),
                            Text(widget.networkMessage, textAlign: TextAlign.center, style: Font.bodyRegular.style(color: ECColor.textColor),),
                          ],
                        )
                      ),
                  );
                }
              );
            default:
              return Container(
                constraints: BoxConstraints(
                  minHeight: 200
                ),
                child: Center(
                  child: LoadingProgressBar(),
                ),
              );
          }
        else
          return Container(
            constraints: BoxConstraints(
              minHeight: 200
            ),
            child: Center(
              child: LoadingProgressBar(),
            ),
          );
        }
      )
    );
  }

  @override
  void dispose() {
    _heightController.close();
    super.dispose();
  }
}